@@ -10,13 +10,13 @@
 #ifndef __XML_TREE_H__
 #define __XML_TREE_H__
 
+#include <stdio.h>
+
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#include <stdio.h>
-
 /*
  * The different element types carried by an XML tree
  *
@@ -427,10 +427,20 @@ int		xmlNodeIsText		(xmlNodePtr node);
 /*
  * Changing the structure
  */
+xmlNodePtr	xmlDocSetRootElement	(xmlDocPtr doc,
+					 xmlNodePtr root);
+void		xmlNodeSetName		(xmlNodePtr cur,
+					 const xmlChar *name);
 xmlNodePtr	xmlAddChild		(xmlNodePtr parent,
 					 xmlNodePtr cur);
+xmlNodePtr	xmlReplaceNode		(xmlNodePtr old,
+					 xmlNodePtr cur);
 xmlNodePtr	xmlAddSibling		(xmlNodePtr cur,
 					 xmlNodePtr elem);
+xmlNodePtr	xmlAddPrevSibling	(xmlNodePtr cur,
+					 xmlNodePtr elem);
+xmlNodePtr	xmlAddNextSibling	(xmlNodePtr cur,
+					 xmlNodePtr elem);
 void		xmlUnlinkNode		(xmlNodePtr cur);
 xmlNodePtr	xmlTextMerge		(xmlNodePtr first,
 					 xmlNodePtr second);
