@@ -12,11 +12,12 @@
 #ifndef __XML_XPATH_H__
 #define __XML_XPATH_H__
 
-#ifdef __cplusplus
-#define extern "C" {
-#endif
 #include "tree.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct xmlXPathParserContext *xmlXPathParserContextPtr;
 
 /*
